/* -LICENSE-START-
** Copyright (c) 2009 Blackmagic Design
**
** Permission is hereby granted, free of charge, to any person or organization
** obtaining a copy of the software and accompanying documentation covered by
** this license (the "Software") to use, reproduce, display, distribute,
** execute, and transmit the Software, and to prepare derivative works of the
** Software, and to permit third-parties to whom the Software is furnished to
** do so, all subject to the following:
** 
** The copyright notices in the Software and this entire statement, including
** the above license grant, this restriction and the following disclaimer,
** must be included in all copies of the Software, in whole or in part, and
** all derivative works of the Software, unless such copies or derivative
** works are solely in the form of machine-executable object code generated by
** a source language processor.
** 
** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
** IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
** DEALINGS IN THE SOFTWARE.
** -LICENSE-END-
*/
// HDLinkLutRWTestDlg.cpp : implementation file
//

#include "stdafx.h"
#include "HDLinkLutRWTest.h"
#include "HDLinkLutRWTestDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CHDLinkArrivalDelegate::CHDLinkArrivalDelegate(CHDLinkLutRWTestDlg* pController)
{
	m_pController = pController;
	m_RefCount = 0;
}

CHDLinkArrivalDelegate::~CHDLinkArrivalDelegate()
{
}

HRESULT	STDMETHODCALLTYPE CHDLinkArrivalDelegate::QueryInterface(REFIID iid, LPVOID *ppv)
{
	HRESULT			result = E_NOINTERFACE;
	
	// Initialise the return result
	*ppv = NULL;
	
	// Obtain the IUnknown interface and compare it the provided REFIID
	if (iid == IID_IUnknown)
	{
		*ppv = this;
		AddRef();
		result = S_OK;
	}
	else if (iid == IID_IHDLinkDeviceArrivalCallback)
	{
		*ppv = (IHDLinkDeviceArrivalCallback*)this;
		AddRef();
		result = S_OK;
	}
	
	return result;
}

ULONG STDMETHODCALLTYPE CHDLinkArrivalDelegate::AddRef(void)
{
	return InterlockedIncrement((LONG*)&m_RefCount);
}

ULONG STDMETHODCALLTYPE CHDLinkArrivalDelegate::Release(void)
{
	int		newRefValue;
	
	newRefValue = InterlockedDecrement((LONG*)&m_RefCount);
	if (newRefValue == 0)
	{
		delete this;
		return 0;
	}
	
	return newRefValue;
}

HRESULT STDMETHODCALLTYPE  CHDLinkArrivalDelegate::HDLinkDeviceArrival(IHDLink *Sender)
{
    m_pController->DeviceAdded(Sender);
	return S_OK;
}

CHDLinkRemovalDelegate::CHDLinkRemovalDelegate(CHDLinkLutRWTestDlg* pController)
{
	m_pController = pController;
	m_RefCount = 0;
}

CHDLinkRemovalDelegate::~CHDLinkRemovalDelegate()
{
}

HRESULT	STDMETHODCALLTYPE CHDLinkRemovalDelegate::QueryInterface(REFIID iid, LPVOID *ppv)
{
	HRESULT			result = E_NOINTERFACE;
	
	// Initialise the return result
	*ppv = NULL;
	
	// Obtain the IUnknown interface and compare it the provided REFIID
	if (iid == IID_IUnknown)
	{
		*ppv = this;
		AddRef();
		result = S_OK;
	}
	else if (iid == IID_IHDLinkDeviceRemovalCallback)
	{
		*ppv = (IHDLinkDeviceRemovalCallback*)this;
		AddRef();
		result = S_OK;
	}
	
	return result;
}

ULONG STDMETHODCALLTYPE CHDLinkRemovalDelegate::AddRef(void)
{
	return InterlockedIncrement((LONG*)&m_RefCount);
}

ULONG STDMETHODCALLTYPE CHDLinkRemovalDelegate::Release(void)
{
	int		newRefValue;
	
	newRefValue = InterlockedDecrement((LONG*)&m_RefCount);
	if (newRefValue == 0)
	{
		delete this;
		return 0;
	}
	
	return newRefValue;
}

HRESULT STDMETHODCALLTYPE  CHDLinkRemovalDelegate::HDLinkDeviceRemoval(IHDLink *Sender)
{
	m_pController->DeviceRemove(Sender, m_RefCon);
	return S_OK;
}

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

	// Dialog Data
	enum { IDD = IDD_ABOUTBOX };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

	// Implementation
protected:
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()

CHDLinkLutRWTestDlg::CHDLinkLutRWTestDlg(CWnd* pParent /*=NULL*/)
: CDialog(CHDLinkLutRWTestDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_pArrivalDelegate	= new  CHDLinkArrivalDelegate(this);		
}

CHDLinkLutRWTestDlg::~CHDLinkLutRWTestDlg()
{
}

void CHDLinkLutRWTestDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LUT_READ_BTN, m_LutReadBtn);
	DDX_Control(pDX, IDC_LUT_LOAD_BTN, m_LutLoadBtn);
	DDX_Control(pDX, IDC_LUT_LOAD_NV_BTN, m_LutLoadNvBtn);
	DDX_Control(pDX, ID_HDLINK_RESET_BTN, m_HDLinkRestBtn);
	DDX_Control(pDX, ID_HDLINK_RESET_NV_BTN, m_HDLinkRestNvBtn);
	DDX_Control(pDX, IDC_HDLINK_DEVS_COMBO, m_HDLinkDevsCombo);
	DDX_Control(pDX, IDC_DEVICE_STATUS_LBL, m_HDLinkDevsStatusLbl);
}

BEGIN_MESSAGE_MAP(CHDLinkLutRWTestDlg, CDialog)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//ON_WM_DEVICECHANGE()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(ID_HDLINK_CONNECT_BTN, OnBnClickedHdlinkConnectBtn)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDC_LUT_READ_BTN, OnBnClickedLutReadBtn)
	ON_BN_CLICKED(IDC_LUT_LOAD_BTN, OnBnClickedLutLoadBtn) 
	ON_WM_CLOSE()
	ON_WM_DESTROY()
	ON_BN_CLICKED(ID_HDLINK_RESET_BTN, OnBnClickedHdlinkResetBtn)
	ON_BN_CLICKED(IDC_LUT_LOAD_NV_BTN, OnBnClickedLutLoadNvBtn)
	ON_BN_CLICKED(ID_HDLINK_RESET_NV_BTN, OnBnClickedHdlinkResetNvBtn)
END_MESSAGE_MAP()

// CHDLinkLutRWTestDlg message handlers

BOOL CHDLinkLutRWTestDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}
	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	m_HDLinkDevsCombo.EnableWindow(false);
	m_HDLinkDevsCombo.SetWindowText("<No HDLink connected>");
	m_HDLinkDevsStatusLbl.SetWindowText("No HDLink connected");
    m_Cur3DLookupTableConfiguration =  NULL;
	UpdateControls(false);
	CoInitialize(NULL);
	if (CoCreateInstance(CLSID_CHDLinkDiscovery, NULL, CLSCTX_ALL, IID_IHDLinkDiscovery, reinterpret_cast<void**>(&m_HDLinkDiscovery)) != S_OK ||
		m_HDLinkDiscovery == NULL)
	{
		return FALSE;
	}
    m_HDLinkDiscovery->InstallDeviceAddedNotification(m_pArrivalDelegate, this->GetSafeHwnd());
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CHDLinkLutRWTestDlg::OnClose()
{
	CDialog::OnClose();
}

void CHDLinkLutRWTestDlg::OnDestroy()
{
    //Release Connected HDLink devices
	for (int index = 0; index < m_HDLinkDevsCombo.GetCount(); index++)
	{
		IHDLink* HDLinkDevice = (IHDLink*)m_HDLinkDevsCombo.GetItemData(index);
		HDLinkDevice->UninstallDeviceRemovalNotification();
	}

	CDialog::OnDestroy();
    if (m_HDLinkDiscovery)
		m_HDLinkDiscovery->Release();
	m_HDLinkDiscovery = NULL;

	if (m_pArrivalDelegate)
		delete m_pArrivalDelegate;
	m_pArrivalDelegate = NULL;

	CoUninitialize();
}

void CHDLinkLutRWTestDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CHDLinkLutRWTestDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CHDLinkLutRWTestDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CHDLinkLutRWTestDlg::OnBnClickedHdlinkConnectBtn()
{
	IHDLink *refHDLink = (IHDLink*)m_HDLinkDevsCombo.GetItemData(m_HDLinkDevsCombo.GetCurSel());
	if (refHDLink == NULL)
		return;
    if (m_Cur3DLookupTableConfiguration != NULL)
		m_Cur3DLookupTableConfiguration->Release();
	m_Cur3DLookupTableConfiguration = NULL;

	if (refHDLink->QueryInterface(IID_IHDLink3DLUTConfiguration, reinterpret_cast<void**>(&m_Cur3DLookupTableConfiguration)) == S_OK)
	{
		UpdateControls(true);
	}
}

void CHDLinkLutRWTestDlg::OnBnClickedOk()
{
	OnOK();
}

void CHDLinkLutRWTestDlg::UpdateControls(bool EnableCtls)
{
	m_LutReadBtn.EnableWindow(EnableCtls);
	m_LutLoadBtn.EnableWindow(EnableCtls);
	m_HDLinkDevsCombo.EnableWindow(EnableCtls);
	m_HDLinkRestBtn.EnableWindow(EnableCtls);
	m_LutLoadNvBtn.EnableWindow(EnableCtls);
	m_HDLinkRestNvBtn.EnableWindow(EnableCtls);
}

void CHDLinkLutRWTestDlg::OnBnClickedLutReadBtn()
{	
	// Read the gamma table
	IHDLink *refHDLink = (IHDLink*)m_HDLinkDevsCombo.GetItemData(m_HDLinkDevsCombo.GetCurSel());
	if (refHDLink != NULL)
	{
		if (m_Cur3DLookupTableConfiguration != NULL)
			m_Cur3DLookupTableConfiguration->Release();
		m_Cur3DLookupTableConfiguration = NULL;
		unsigned long cubeDim = 0;
	    LOOKUPTABLE_COLOR*				pReadLut3DTable = NULL;
		BSTR							TableName = OLESTR("");
		if (refHDLink->QueryInterface(IID_IHDLink3DLUTConfiguration, reinterpret_cast<void**>(&m_Cur3DLookupTableConfiguration)) == S_OK)
		{
			m_Cur3DLookupTableConfiguration->ReadActiveTable(&pReadLut3DTable, &cubeDim, &TableName);

		}
	}
}

void CHDLinkLutRWTestDlg::OnBnClickedLutLoadBtn()
{
    // Load the gamma table
	IHDLink *refHDLink = (IHDLink*)m_HDLinkDevsCombo.GetItemData(m_HDLinkDevsCombo.GetCurSel());
	if (refHDLink != NULL)
	{
		if (m_Cur3DLookupTableConfiguration != NULL)
			m_Cur3DLookupTableConfiguration->Release();
		m_Cur3DLookupTableConfiguration = NULL;
		BSTR	tableName = OLESTR("New LUT");
		if (refHDLink->QueryInterface(IID_IHDLink3DLUTConfiguration, reinterpret_cast<void**>(&m_Cur3DLookupTableConfiguration)) == S_OK)
		{
			m_Cur3DLookupTableConfiguration->WriteTable(Lut3DTable, kLutCubePoints1D, tableName, FALSE);
		}
	}
}

// This function is called when a device notification event (that we have installed) occurs
BOOL	CHDLinkLutRWTestDlg::OnDeviceChange(UINT nEventType, DWORD_PTR dwData)
{
	return true;
}

void CHDLinkLutRWTestDlg::DeviceAdded(IHDLink *Sender)
{
	CComBSTR					HDLinkModelName;
	CString						HDLinkName;
	if (Sender->GetModelName(&HDLinkModelName) != S_OK)
		HDLinkName = "Unknown HDLink";
	HDLinkName = HDLinkModelName;
	int index = m_HDLinkDevsCombo.AddString(HDLinkName);
	m_HDLinkDevsCombo.SetItemDataPtr(index, Sender);
    CHDLinkRemovalDelegate* removalDelegate = new CHDLinkRemovalDelegate(this);
    removalDelegate->m_RefCon = (void*)index;
	Sender->InstallDeviceRemovalNotification((IHDLinkDeviceRemovalCallback*)removalDelegate, this->GetSafeHwnd());
    m_HDLinkDevsCombo.SetCurSel(index);
	m_HDLinkDevsCombo.EnableWindow(true);
	UpdateControls(true);
	CString DevString;
	DevString.Format("HDLinks connected : %d", m_HDLinkDevsCombo.GetCount());
	m_HDLinkDevsStatusLbl.SetWindowText(DevString);
}

void CHDLinkLutRWTestDlg::DeviceRemove(IHDLink *Sender, void* refCon)
{
	int index = (int)refCon;
	if (!(index < 0))
		m_HDLinkDevsCombo.GetItemData(index);
	m_HDLinkDevsCombo.DeleteString(index);
    Sender->UninstallDeviceRemovalNotification();

	if (m_HDLinkDevsCombo.GetCount() == 0)
	{
		m_HDLinkDevsCombo.EnableWindow(false);
		UpdateControls(false);
		m_HDLinkDevsCombo.SetWindowText("<No HDLink connected>");
	}
	CString DevString;
	DevString.Format("HDLinks connected : %d", m_HDLinkDevsCombo.GetCount());
	m_HDLinkDevsStatusLbl.SetWindowText(DevString);
}

void CHDLinkLutRWTestDlg::OnBnClickedHdlinkResetBtn()
{
	// Reset the gamma table
	IHDLink *refHDLink = (IHDLink*)m_HDLinkDevsCombo.GetItemData(m_HDLinkDevsCombo.GetCurSel());
	if (refHDLink != NULL)
	{
		if (m_Cur3DLookupTableConfiguration != NULL)
			m_Cur3DLookupTableConfiguration->Release();
		m_Cur3DLookupTableConfiguration = NULL;
		BSTR	tableName = OLESTR("New LUT");
		if (refHDLink->QueryInterface(IID_IHDLink3DLUTConfiguration, reinterpret_cast<void**>(&m_Cur3DLookupTableConfiguration)) == S_OK)
		{
			m_Cur3DLookupTableConfiguration->ResetTable(FALSE);
		}
	}
}

void CHDLinkLutRWTestDlg::OnBnClickedLutLoadNvBtn()
{
	// Load the gamma table to NV memory
	IHDLink *refHDLink = (IHDLink*)m_HDLinkDevsCombo.GetItemData(m_HDLinkDevsCombo.GetCurSel());
	if (refHDLink != NULL)
	{
		if (m_Cur3DLookupTableConfiguration != NULL)
			m_Cur3DLookupTableConfiguration->Release();
		m_Cur3DLookupTableConfiguration = NULL;
		BSTR	tableName = OLESTR("New LUT");
		if (refHDLink->QueryInterface(IID_IHDLink3DLUTConfiguration, reinterpret_cast<void**>(&m_Cur3DLookupTableConfiguration)) == S_OK)
		{
			m_Cur3DLookupTableConfiguration->WriteTable(Lut3DTable, kLutCubePoints1D, tableName, TRUE);
		}
	}
}

void CHDLinkLutRWTestDlg::OnBnClickedHdlinkResetNvBtn()
{
	// Reset gamma table to NV memory
	IHDLink *refHDLink = (IHDLink*)m_HDLinkDevsCombo.GetItemData(m_HDLinkDevsCombo.GetCurSel());
	if (refHDLink != NULL)
	{
		if (m_Cur3DLookupTableConfiguration != NULL)
			m_Cur3DLookupTableConfiguration->Release();
		m_Cur3DLookupTableConfiguration = NULL;
		BSTR	tableName = OLESTR("New LUT");
		if (refHDLink->QueryInterface(IID_IHDLink3DLUTConfiguration, reinterpret_cast<void**>(&m_Cur3DLookupTableConfiguration)) == S_OK)
		{
			m_Cur3DLookupTableConfiguration->ResetTable(TRUE);
		}
	}
}
